#include "configuration.h"
#include "rng.h"
#include "swarm.h"
#include "problem.h"
#include "pso.h"
#include "frankenstein.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Frankenstein::Frankenstein(Configuration* config, Swarm* swarm, Problem* problem) : PSO(config,swarm,problem){}
	 
Frankenstein::~Frankenstein(){}

bool Frankenstein::notDone(){	
	if(p->getFunctionEvaluations() >= config->getMaxFES() || p->getBestSolution() <= DELTA)
		return false;	
	return true;	
}

void Frankenstein::run(){

	//FI
	const double* velocityVector;
	const double* positionVector;
	const double* neighborBestPositionVector;
	double newVelocity[config->getProblemDimension()];
	double newPosition[config->getProblemDimension()];

	std::vector<int> neighbors;

	Particle* particle;		

	unsigned int iteration=0;
	double inertiaWeight=0;
	int removals;
	int topologyUpdatePeriod=(int)floor((double)config->getTopologySchedule()/(config->getPopulationSize()-3)); 
	// Alternatively, you can use the ceiling function. If you use the floor function, all changes will be made before or at config->getTopologySchedule() iterations, 
	// if you use the function ceil, the changes will be done after or at config->getTopologySchedule() iterations. This depends on the size of the 
	// population and the schedules chosen.
	int esteps = 0;
	RNG::initializePermutation(config->getPopulationSize());
	//Main loop
	while(notDone()){
				
		//Evaluation		
		for(int i=0;i<s->getSize();i++)	
			(s->getParticle(i))->evaluateCurrentPosition();	

		//Topology update
		if( (iteration > 0) && (esteps < config->getPopulationSize()-3) && (iteration%topologyUpdatePeriod == 0) ){
			removals = 0;
// 			cout << " Iteration: "<<iteration << " Target: " << config->getPopulationSize()-(2+esteps) << endl;						
			RNG::shufflePermutation();
			while(removals < config->getPopulationSize()-(2+esteps)){												
				for(unsigned int i=0;i<config->getPopulationSize();i++){
					int particleIndex = RNG::getPermutationElement(i);
					if( (s->getParticle(particleIndex))->getNeighborhoodSize() > 3 ){ //3 because a particle is a neighbor to itself
						int neighborID = (s->getParticle(particleIndex))->getRandomNonAdjacentNeighborID();

// 						cout << "Erasing edge " << particleIndex << " <---> " << neighborID << endl;

						(s->getParticle(particleIndex))->eraseNeighborbyID(neighborID);
						(s->getParticle(neighborID))->eraseNeighborbyID(particleIndex);
		
						removals++;
					}
					if(removals == config->getPopulationSize()-(2+esteps))
						break;
				}				
			}			
			esteps++;		
// 			cout << "Removals " << removals << endl;							
		}			
	
		//Decreasing inertia weight
		if(iteration <= config->getIWSchedule())
			inertiaWeight = ((double)(config->getIWSchedule() - iteration)/config->getIWSchedule())*(config->getInitialIW() - config->getFinalIW()) + config->getFinalIW();			
		else
			inertiaWeight = config->getFinalIW();
		
		for(int i=0;i<s->getSize();i++){

			particle = s->getParticle(i);
										
// 			cout << "Particle "<<particle->getID()<< " using FIPS" <<endl;

			neighbors = particle->getNeighbors();

			//Update velocity & positions
			velocityVector = particle->getCurrentVelocity();
			positionVector = particle->getCurrentPosition();				
					
// 			cout << " Particle " << particle->getID() << " neighbors ";
// 			for(int k=0;k<neighbors.size();k++) 
// 			cout << (s->getParticle(neighbors.at(k)))->getID() << " ";
// 			cout << endl;

			for(unsigned int j=0;j<config->getProblemDimension();j++){

				double sum=0.0;						
				for(unsigned int k=0;k<neighbors.size();k++){							
					neighborBestPositionVector = (s->getParticle(neighbors.at(k)))->getBestPosition();
					sum += RNG::randVal(0.0,(4.0/(double)neighbors.size()))*( neighborBestPositionVector[j] - positionVector[j] );	
				}

				//Velocity update					
				newVelocity[j] = inertiaWeight*velocityVector[j] + sum;
						
				//Velocity restriction
				newVelocity[j] = limitVelocity(newVelocity[j]); //One of several posible velocity control mechanisms	
				newPosition[j] = positionVector[j] + newVelocity[j];
			}
					
			particle->setCurrentVelocity(newVelocity);
			particle->setCurrentPosition(newPosition);										

// 			particle->evaluateCurrentPosition();
		}

		iteration++;
	}
	RNG::deallocatePermutation();
}
