#include "rng.h"
#include "configuration.h"
#include "problem.h"
#include "particle.h"

#include <cstdlib>
#include <cfloat>
#include <iostream>
#include <vector>

using namespace std;

Particle::Particle(Configuration* config,Problem* problem,int identifier) {
	this->config = config;	
	this->problem = problem;	
	id=identifier;
	
	currentPosition = new double[config->getProblemDimension()]; 	
	currentVelocity = new double[config->getProblemDimension()]; 	
	bestPosition = new double[config->getProblemDimension()]; 
	
	for(unsigned int i=0;i<config->getProblemDimension();i++){
		currentPosition[i]=RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());
		currentVelocity[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange()) - currentPosition[i];
		bestPosition[i]=currentPosition[i];
	}

	if(config->isMinimization())
		currentValue=bestValue=DBL_MAX;		
	else
		currentValue=bestValue=-DBL_MAX;					

	//Default topology: FC
	for(unsigned int i=0;i<config->getPopulationSize();i++)				
		neighbors.push_back(i);		
}

Particle::~Particle(){
	neighbors.clear();
	delete[] currentPosition;
	delete[] currentVelocity;	
	delete[] bestPosition;
}

int Particle::getID(){
	return id;
}

int Particle::getDimension(){ 
	return config->getProblemDimension(); 
}

std::vector<int> Particle::getNeighbors(){
	return neighbors;
}

unsigned int Particle::getNeighborhoodSize(){
	return neighbors.size();
}

void Particle::eraseNeighborhood(){
	neighbors.clear();
}

void Particle::addNeighbor(int neighborID){
	neighbors.push_back(neighborID);
}

int Particle::getNeighborIDbyIndex(int neighborIndex){
	return neighbors.at(neighborIndex);
}

void Particle::eraseNeighborbyIndex(int neighborIndex){
	neighbors.erase(neighbors.begin()+neighborIndex);	
}

void Particle::eraseNeighborbyID(int nid){
	for(unsigned int i=0;i<neighbors.size();i++){
		if(nid == neighbors.at(i)){
			neighbors.erase(neighbors.begin()+i);
			break;
		}
	}	
}

int Particle::getRandomNonAdjacentNeighborID(){
	int randomIndex,randomID;
	if(neighbors.size()>3){
		while(true){
			randomIndex = (int)floor(RNG::randVal(0.0,(double)neighbors.size()));
			randomID = neighbors.at(randomIndex);
		
			if(id==0){
				if(randomID != id && randomID != id+1 && randomID != config->getPopulationSize()-1)			
					return randomID;
				else 
					continue;
			}
			else{
				if(id==config->getPopulationSize()-1){
					if(randomID != id && randomID != 0 && randomID != id-1)			
						return randomID;
					else 
						continue;
				}
				else{
					if(randomID != id && randomID != id+1 && randomID != id-1)			
						return randomID;
					else 
						continue;
				}
			}		
		}
	}
	else		
		return -1;			
}

void Particle::evaluateCurrentPosition(){
	currentValue = problem->getFunctionValue(currentPosition);	
	//Update memory			
	if(config->isMinimization()){				
		if(currentValue <= bestValue)
			updateBestPosition(currentValue);				
	}
	else{						
		if(currentValue >= bestValue)
			updateBestPosition(currentValue);					
	}
}

const double* Particle::getCurrentPosition(){
	return currentPosition;
}
void Particle::setCurrentPosition(double* newPosition){
	for(unsigned int i=0;i<config->getProblemDimension();i++)
		currentPosition[i]=newPosition[i];	
}

const double* Particle::getCurrentVelocity(){
	return currentVelocity;
}
void Particle::setCurrentVelocity(double* newVelocity){
	for(unsigned int i=0;i<config->getProblemDimension();i++)
		currentVelocity[i]=newVelocity[i];
}

const double* Particle::getBestPosition(){
	return bestPosition;
}
void Particle::setBestPosition(double* newBestPosition){
	for(unsigned int i=0;i<config->getProblemDimension();i++)
		bestPosition[i]=newBestPosition[i];
}
void Particle::updateBestPosition(double newBest){
	setBestValue(newBest);
	for(unsigned int i=0;i<config->getProblemDimension();i++)
		bestPosition[i]=currentPosition[i];
}


double Particle::getCurrentValue(){
	return currentValue;
}
void Particle::setCurrentValue(double newValue){
	currentValue=newValue;
}

double Particle::getBestValue(){
	return bestValue;
}
void Particle::setBestValue(double newBestValue){	
	bestValue=newBestValue;
}

void Particle::print(){	
	unsigned int i;
	cout << endl;
	cout << "Particle " << id << endl;
	cout << "Current position : " <<endl;
	for(i=0;i<config->getProblemDimension();i++)
		cout << currentPosition[i] << " ";
	cout << endl;
	cout << "Current velocity : " <<endl;
	for(i=0;i<config->getProblemDimension();i++)
		cout << currentVelocity[i] << " ";
	cout << endl;
	cout << "Current best position : " <<endl;
	for(i=0;i<config->getProblemDimension();i++)
		cout << bestPosition[i] << " ";
	cout << endl;
	cout << "Current value = " << currentValue << endl;
	cout << "Best value = " << bestValue << endl;
	cout << "Neighbors : " <<endl;
	for(i=0;i<neighbors.size();i++)
		cout << neighbors[i] << " ";
	cout << endl;
}
