#include "rng.h"
#include "configuration.h"
#include "problem.h"
#include "particle.h"
#include "swarm.h"

#include <iostream>
#include <vector>
#include <cstdlib>
#include <cmath>

using namespace std;

 Swarm::Swarm(Configuration* config,Problem* problem){		
	this->config=config;			
	for(numberOfParticles=0;numberOfParticles < config->getPopulationSize(); numberOfParticles++)
		particles.push_back(new Particle(config,problem,numberOfParticles)); 				
}

Swarm::~Swarm(){	
	for(unsigned int i=0;i<particles.size();i++)
		delete particles.at(i);		
	particles.clear();
}

int Swarm::getSize(){
	return particles.size();
}

Particle* Swarm::getParticle(int id){	
	if(id < particles.size() && id >= 0)
		return particles.at(id);
	else{
		cerr << "Trying to access an inexistent particle" << endl;
		exit(-1);
	}
}

Particle* Swarm::getBestNeighborOf(Particle* p){
	unsigned int i;
	double best,current;
	Particle* bestParticle;
	Particle* currentParticle;
	std::vector<int> neighbors = p->getNeighbors();

	for(i=0;i<neighbors.size();i++){
		if(i==0){
			bestParticle = particles.at(neighbors.at(i));
			best = bestParticle->getBestValue();
		}
		else{
			currentParticle = particles.at(neighbors.at(i));
			current = currentParticle->getBestValue();	

			if(config->isMinimization()){
				if(current < best){
					bestParticle = currentParticle;
					best = current;
				}
			}
			else{
				if(current > best){
					bestParticle = currentParticle;
					best = current;
				}
			}

		}
	}
	return bestParticle;
}

double Swarm::getSolutionValue(){
	double temp,solution;
	for(unsigned int i=0;i<particles.size();i++){
		temp = (particles.at(i))->getBestValue();
		if(i==0)
			solution=temp;		
		else{
			if(config->isMinimization()){
				if(temp < solution)
					solution=temp;	
			}
			else{
				if(temp > solution)
					solution=temp;		
			}
		}
	}
	return solution;
}

const double* Swarm::getSolution(){
	double temp,solution;
	int index;
	for(unsigned int i=0;i<particles.size();i++){
		temp = (particles.at(i))->getBestValue();
		if(i==0){
			solution=temp;		
			index = i;
		}
		else {
			if(config->isMinimization()){
				if(temp < solution){
					solution=temp;		
					index = i;
				}
			}
			else{
				if(temp > solution){
					solution=temp;		
					index = i;
				}
			}
		}
	}	
	return (particles.at(index))->getBestPosition();	
}

Particle* Swarm::getBestParticle(){
	double temp,solution;
	int index;
	for(unsigned int i=0;i<particles.size();i++){
		temp = (particles.at(i))->getBestValue();
		if(i==0){
			solution=temp;		
			index = i;
		}
		else {
			if(config->isMinimization()){
				if(temp < solution){
					solution=temp;		
					index = i;
				}
			}
			else{
				if(temp > solution){
					solution=temp;		
					index = i;
				}
			}
		}
	}	
	return particles.at(index);	
}

Particle* Swarm::getRandomParticle(){
	double index = RNG::randVal(0.0,(double)particles.size());
	return particles.at((int)floor(index));
}
		
double Swarm::getAvgDist2Centroid(){
	unsigned int i, j;
	int dimension = (particles.at(0))->getDimension();
	double* centroid = (double*)calloc(dimension,sizeof(double));
	const double* temp;	
	double acc,distance=0.0;

	for(i=0;i<particles.size();i++){
		temp = (particles.at(i))->getCurrentPosition();
		for(j=0;j<dimension;j++)
			centroid[j] += temp[j];		
	}
	for(j=0;j<dimension;j++)
		centroid[j] /= particles.size();	
			
	for(i=0;i<particles.size();i++){
		temp = (particles.at(i))->getCurrentPosition();
		acc=0.0;
		for(j=0;j<dimension;j++)
			acc+=(centroid[j]-temp[j])*(centroid[j]-temp[j]);		
		distance += sqrt(acc);
	}		
	free(centroid);
	return acc/particles.size();
}
	
double Swarm::getAvgSpeed(){
	unsigned int i, j;
	int dimension = (particles.at(0))->getDimension();
	const double* temp;	
	double acc,speed=0.0;

	for(i=0;i<particles.size();i++){
		temp = (particles.at(i))->getCurrentVelocity();
		acc=0.0;
		for(j=0;j<dimension;j++)		
			acc += temp[j]*temp[j];
		speed += sqrt(acc);
	}
	return speed/=particles.size();
}

void Swarm::print(){
	Particle* particle;
	cout << "Swarm" << endl;
	for(unsigned int i=0;i<particles.size();i++){	
		particle=particles.at(i);
		particle->print();
	}
}
